/// <reference types="node" />
import * as os from 'os';
import * as minimistType from 'minimist';
import { AliasedMap } from './utils/object';
export declare type NetworkInterface = {
    device: string;
} & os.NetworkInterfaceInfo;
export interface TerminalInfo {
    tty: boolean;
    ci: boolean;
}
export declare type ParsedArg = string | boolean | null | undefined | string[];
export declare type Validator = (input?: string, key?: string) => true | string;
export declare type CommandLineInputs = string[];
export interface CommandLineOptions extends minimistType.ParsedArgs {
    [arg: string]: ParsedArg;
}
export declare type CommandOptionType = StringConstructor | BooleanConstructor;
export interface CommandMetadataInput {
    name: string;
    summary: string;
    validators?: Validator[];
    private?: boolean;
}
export declare type MetadataGroup = string | number | symbol;
export interface CommandMetadataOption {
    name: string;
    summary: string;
    type?: CommandOptionType;
    default?: ParsedArg;
    aliases?: string[];
    groups?: MetadataGroup[];
}
export interface HydratedCommandOption {
    type: CommandOptionType;
    default: ParsedArg;
    aliases: string[];
}
export interface HydratedParseArgsOptions extends minimistType.Opts {
    string: string[];
    boolean: string[];
    alias: {
        [key: string]: string[];
    };
    default: {
        [key: string]: ParsedArg;
    };
}
export interface Metadata {
    name: string;
    summary: string;
    description?: string;
    groups?: MetadataGroup[];
    aliases?: string[];
}
export interface CommandMetadata<I = CommandMetadataInput, O = CommandMetadataOption> extends Metadata {
    exampleCommands?: string[];
    inputs?: I[];
    options?: O[];
}
export interface CommandInstanceInfo<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    location: NamespaceLocateResult<C, N, M, I, O>;
    env: {
        [key: string]: string;
    };
    executor: IExecutor<C, N, M, I, O>;
}
export interface ICommand<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    namespace: N;
    getMetadata(runinfo?: Partial<CommandInstanceInfo<C, N, M, I, O>>): Promise<M>;
    run(inputs: CommandLineInputs, options: CommandLineOptions, runinfo?: Partial<CommandInstanceInfo<C, N, M, I, O>>): Promise<void>;
    validate(argv: CommandLineInputs): Promise<void>;
}
export declare type CommandMapGetter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> = () => Promise<C>;
export declare type NamespaceMapGetter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> = () => Promise<N>;
export declare type ICommandMap<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> = AliasedMap<string, CommandMapGetter<C, N, M, I, O>>;
export declare type INamespaceMap<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> = AliasedMap<string, NamespaceMapGetter<C, N, M, I, O>>;
export interface INamespace<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    parent: N | undefined;
    getMetadata(): Promise<NamespaceMetadata>;
    getNamespaces(): Promise<INamespaceMap<C, N, M, I, O>>;
    getCommands(): Promise<ICommandMap<C, N, M, I, O>>;
    locate(argv: string[]): Promise<NamespaceLocateResult<C, N, M, I, O>>;
    getCommandMetadataList(): Promise<ReadonlyArray<HydratedCommandMetadata<C, N, M, I, O>>>;
}
export declare type CommandPathItem<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> = [string, C | N];
export interface NamespaceLocateResult<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly obj: C | N;
    readonly args: ReadonlyArray<string>;
    readonly path: ReadonlyArray<CommandPathItem<C, N, M, I, O>>;
}
export declare type HydratedCommandMetadata<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> = M & {
    readonly command: C;
    readonly namespace: N;
    readonly path: ReadonlyArray<CommandPathItem<C, N, M, I, O>>;
    readonly aliases: ReadonlyArray<string>;
};
export interface NamespaceMetadata extends Metadata {
}
export interface HydratedNamespaceMetadata extends NamespaceMetadata {
    aliases: string[];
}
export interface IExecutor<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly namespace: N;
    execute(argv: string[], env: {
        [key: string]: string;
    }): Promise<void>;
    run(command: C, cmdargs: string[], runinfo?: Partial<CommandInstanceInfo<C, N, M, I, O>>): Promise<void>;
}
export interface PackageJson {
    name: string;
    version: string;
    main?: string;
    description?: string;
    bin?: {
        [key: string]: string;
    };
    scripts?: {
        [key: string]: string;
    };
    dependencies?: {
        [key: string]: string;
    };
    devDependencies?: {
        [key: string]: string;
    };
}
export interface Validators {
    required: Validator;
    email: Validator;
    numeric: Validator;
    url: Validator;
    slug: Validator;
}
export interface ValidationError {
    key: string;
    message: string;
    validator: Validator;
}
