"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const array_1 = require("../utils/array");
const format_1 = require("../utils/format");
const colors_1 = require("./colors");
const validators_1 = require("./validators");
const DEFAULT_DOTS_WIDTH = 25;
class HelpFormatter {
    constructor({ colors }) {
        this.colors = colors ? colors : colors_1.DEFAULT_COLORS;
    }
}
exports.HelpFormatter = HelpFormatter;
class NamespaceHelpFormatter extends HelpFormatter {
    constructor({ location, namespace, colors }) {
        super({ colors });
        this.dotswidth = DEFAULT_DOTS_WIDTH;
        this.location = location;
        this.namespace = namespace;
    }
    getNamespaceMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this._metadata) {
                this._metadata = yield this.namespace.getMetadata();
            }
            return this._metadata;
        });
    }
    getNamespaceFullName() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this._fullName) {
                this._fullName = this.location.path.map(([p]) => p).join(' ');
            }
            return this._fullName;
        });
    }
    formatHeader() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong, input } = this.colors;
            const fullName = yield this.getNamespaceFullName();
            const summary = yield this.formatSummary();
            const description = yield this.formatDescription();
            return (`\n  ${strong(`${input(fullName)}${summary}`)}` +
                (description ? `\n\n    ${description}` : '') + '\n');
        });
    }
    formatSummary() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const fullName = yield this.getNamespaceFullName();
            const metadata = yield this.getNamespaceMetadata();
            const summary = ((this.formatBeforeSummary ? yield this.formatBeforeSummary() : '') +
                metadata.summary +
                (this.formatAfterSummary ? yield this.formatAfterSummary() : ''));
            const wrappedSummary = format_1.wordWrap(summary, { indentation: fullName.length + 5 });
            return wrappedSummary ? ` - ${wrappedSummary}` : '';
        });
    }
    formatDescription() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metadata = yield this.getNamespaceMetadata();
            if (!metadata.description) {
                return '';
            }
            return format_1.wordWrap(metadata.description.trim(), { indentation: 4 });
        });
    }
    getExtraOptions() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return [];
        });
    }
    formatUsage() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong, weak, input } = this.colors;
            const fullName = yield this.getNamespaceFullName();
            const options = ['--help', ...(yield this.getExtraOptions())];
            const usageLines = [
                `<command> ${weak('[<args>]')} ${options.map(opt => weak('[' + opt + ']')).join(' ')} ${weak('[options]')}`,
            ];
            return (`\n  ${strong('Usage')}:` +
                `\n\n    ${usageLines.map(u => `${weak('$')} ${input(fullName + ' ' + u)}`).join('\n    ')}\n`);
        });
    }
    formatCommands() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const commands = yield this.namespace.getCommandMetadataList();
            return this.formatCommandGroup('Commands', commands);
        });
    }
    formatCommandGroup(title, commands) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong } = this.colors;
            const filterCallback = this.filterCommandCallback;
            const filteredCommands = filterCallback ? yield array_1.filter(commands, (cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () { return filterCallback(cmd); })) : commands;
            const [cmdDetails, nsDetails] = yield Promise.all([
                this.getListOfCommandDetails(filteredCommands.filter(cmd => cmd.namespace === this.namespace)),
                this.getListOfNamespaceDetails(filteredCommands.filter(cmd => cmd.namespace !== this.namespace)),
            ]);
            const details = [...cmdDetails, ...nsDetails];
            if (details.length === 0) {
                return '';
            }
            details.sort();
            return (`\n  ${strong(title)}:` +
                `\n\n    ${details.join('\n    ')}\n`);
        });
    }
    getListOfCommandDetails(commands) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak, input } = this.colors;
            const fullCmd = commands.map(cmd => cmd.path.map(([p]) => p).join(' '));
            const fillStringArray = format_1.generateFillSpaceStringList(fullCmd, this.dotswidth, weak('.'));
            const formattedCommands = yield Promise.all(commands.map((cmd, index) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const summary = ((this.formatBeforeCommandSummary ? yield this.formatBeforeCommandSummary(cmd) : '') +
                    cmd.summary +
                    (yield this.formatAfterCommandSummary(cmd)));
                const wrappedSummary = format_1.wordWrap(summary, { indentation: this.dotswidth + 6 });
                return `${input(cmd.path.map(p => p[0]).join(' '))}${wrappedSummary ? ' ' + fillStringArray[index] + ' ' + wrappedSummary : ''}`;
            })));
            return formattedCommands;
        });
    }
    getListOfNamespaceDetails(commands) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak, input } = this.colors;
            const summaries = new Map();
            const grouped = new Map();
            yield Promise.all(commands.map((cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const nsmeta = yield cmd.namespace.getMetadata();
                const aliases = [];
                if (cmd.namespace.parent) {
                    const siblings = yield cmd.namespace.parent.getNamespaces();
                    aliases.push(...(siblings.getAliases().get(nsmeta.name) || []).filter((a) => typeof a === 'string'));
                }
                summaries.set(nsmeta.name, nsmeta.summary);
                let entry = grouped.get(nsmeta.name);
                if (!entry) {
                    entry = { meta: Object.assign({}, nsmeta, { aliases }), commands: [] };
                    grouped.set(nsmeta.name, entry);
                }
                entry.commands.push(cmd);
            })));
            const entries = [...grouped.entries()];
            const fillStringArray = format_1.generateFillSpaceStringList(entries.map(([name]) => name + ' <subcommand>'), this.dotswidth, weak('.'));
            const formattedNamespaces = yield Promise.all(entries.map(([name, { meta, commands }], i) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const summary = ((this.formatBeforeNamespaceSummary ? yield this.formatBeforeNamespaceSummary(meta, commands) : '') +
                    summaries.get(name) +
                    (yield this.formatAfterNamespaceSummary(meta, commands)));
                const wrappedSummary = format_1.wordWrap(summary, { indentation: this.dotswidth + 6 });
                return `${input(name + ' <subcommand>')}${wrappedSummary ? ' ' + fillStringArray[i] + ' ' + wrappedSummary : ''}`;
            })));
            return formattedNamespaces;
        });
    }
    /**
     * Insert text that appears after a commands's summary.
     *
     * @param meta: The metadata of the command.
     */
    formatAfterCommandSummary(meta) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak, input } = this.colors;
            const aliases = meta.aliases.length > 0 ? weak('(alias' + (meta.aliases.length === 1 ? '' : 'es') + ': ') + meta.aliases.map(a => input(a)).join(', ') + weak(')') : '';
            return aliases ? ` ${aliases}` : '';
        });
    }
    /**
     * Insert text that appears after a namespace's summary.
     *
     * @param meta The metadata of the namespace.
     * @param commands An array of the metadata of the namespace's commands.
     */
    formatAfterNamespaceSummary(meta, commands) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak, input } = this.colors;
            const subcommands = commands.length > 0 ? `${weak('(subcommands:')} ${commands.map(c => input(c.name)).join(', ')}${weak(')')}` : '';
            const aliases = meta.aliases.length > 0 ? `${weak('(alias' + (meta.aliases.length === 1 ? '' : 'es') + ': ') + meta.aliases.map(a => input(a)).join(', ') + weak(')')}` : '';
            return `${subcommands ? ` ${subcommands}` : ''}${aliases ? ` ${aliases}` : ''}`;
        });
    }
    format() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return ((yield this.formatHeader()) +
                (yield this.formatUsage()) +
                (yield this.formatCommands()) +
                '\n');
        });
    }
}
exports.NamespaceHelpFormatter = NamespaceHelpFormatter;
class CommandHelpFormatter extends HelpFormatter {
    constructor({ location, command, colors }) {
        super({ colors });
        this.dotswidth = DEFAULT_DOTS_WIDTH;
        this.location = location;
        this.command = command;
    }
    getCommandMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this._metadata) {
                this._metadata = yield this.command.getMetadata({ location: this.location });
            }
            return this._metadata;
        });
    }
    getCommandFullName() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this._fullName) {
                this._fullName = this.location.path.map(([p]) => p).join(' ');
            }
            return this._fullName;
        });
    }
    formatHeader() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong, input } = this.colors;
            const fullName = yield this.getCommandFullName();
            const summary = yield this.formatSummary();
            const description = yield this.formatDescription();
            return (`\n  ${strong(`${input(fullName)}${summary}`)}` +
                (description ? `\n\n    ${description}` : '') + '\n');
        });
    }
    formatSummary() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const fullName = yield this.getCommandFullName();
            const metadata = yield this.getCommandMetadata();
            const summary = ((this.formatBeforeSummary ? yield this.formatBeforeSummary() : '') +
                metadata.summary +
                (this.formatAfterSummary ? yield this.formatAfterSummary() : ''));
            const wrappedSummary = format_1.wordWrap(summary, { indentation: fullName.length + 5 });
            return wrappedSummary ? ` - ${wrappedSummary}` : '';
        });
    }
    formatDescription() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metadata = yield this.getCommandMetadata();
            if (!metadata.description) {
                return '';
            }
            return format_1.wordWrap(metadata.description.trim(), { indentation: 4 });
        });
    }
    formatInlineInput(input) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (input.validators && input.validators.includes(validators_1.validators.required)) {
                return '<' + input.name + '>';
            }
            return '[<' + input.name + '>]';
        });
    }
    formatUsage() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong, weak, input } = this.colors;
            const fullName = yield this.getCommandFullName();
            const metadata = yield this.getCommandMetadata();
            const filterCallback = this.filterOptionCallback;
            const options = metadata.options ? metadata.options : [];
            const filteredOptions = filterCallback ? yield array_1.filter(options, (opt) => tslib_1.__awaiter(this, void 0, void 0, function* () { return filterCallback(opt); })) : options;
            const formattedInputs = metadata.inputs ? yield Promise.all(metadata.inputs.map((input) => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.formatInlineInput(input); }))) : [];
            return (`\n  ${strong('Usage')}:` +
                `\n\n    ${weak('$')} ${input(fullName + (formattedInputs ? ' ' + formattedInputs.join(' ') : ''))}${filteredOptions.length > 0 ? ' ' + input('[options]') : ''}\n`);
        });
    }
    formatInputs() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong, weak, input } = this.colors;
            const metadata = yield this.getCommandMetadata();
            const inputs = metadata.inputs ? metadata.inputs : [];
            if (inputs.length === 0) {
                return '';
            }
            const fillStrings = format_1.generateFillSpaceStringList(inputs.map(input => input.name), this.dotswidth, weak('.'));
            const inputLineFn = ({ name, summary }, index) => {
                const optionList = input(`${name}`);
                const wrappedSummary = format_1.wordWrap(summary, { indentation: this.dotswidth + 6 });
                return `${optionList} ${fillStrings[index]} ${wrappedSummary}`;
            };
            return (`\n  ${strong('Inputs')}:` +
                `\n\n    ${inputs.map(inputLineFn).join('\n    ')}\n`);
        });
    }
    formatOptionLine(opt) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak, input } = this.colors;
            const showInverse = opt.type === Boolean && opt.default === true && opt.name.length > 1;
            const optionList = (showInverse ? input(`--no-${opt.name}`) : input(`-${opt.name.length > 1 ? '-' : ''}${opt.name}`)) +
                (!showInverse && opt.aliases && opt.aliases.length > 0 ? ', ' +
                    opt.aliases
                        .map(alias => input(`-${alias}`))
                        .join(', ') : '');
            const optionListLength = format_1.stringWidth(optionList);
            const fullLength = optionListLength > this.dotswidth ? optionListLength + 1 : this.dotswidth;
            const fullDescription = ((this.formatBeforeOptionSummary ? yield this.formatBeforeOptionSummary(opt) : '') +
                opt.summary +
                (yield this.formatAfterOptionSummary(opt)));
            const wrappedDescription = format_1.wordWrap(fullDescription, { indentation: this.dotswidth + 6 });
            return `${optionList} ${weak('.').repeat(fullLength - optionListLength)} ${wrappedDescription}`;
        });
    }
    formatAfterOptionSummary(opt) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.formatOptionDefault(opt);
        });
    }
    formatOptionDefault(opt) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { weak, input } = this.colors;
            if (typeof opt.default === 'string') {
                return weak(' (default: ') + input(opt.default) + weak(')');
            }
            else {
                return '';
            }
        });
    }
    formatOptions() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metadata = yield this.getCommandMetadata();
            const options = metadata.options ? metadata.options : [];
            return this.formatOptionsGroup('Options', options);
        });
    }
    formatOptionsGroup(title, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong } = this.colors;
            const filterCallback = this.filterOptionCallback;
            const filteredOptions = filterCallback ? yield array_1.filter(options, (opt) => tslib_1.__awaiter(this, void 0, void 0, function* () { return filterCallback(opt); })) : options;
            if (filteredOptions.length === 0) {
                return '';
            }
            const formattedOptions = yield Promise.all(filteredOptions.map((option) => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.formatOptionLine(option); })));
            return (`\n  ${strong(title)}:` +
                `\n\n    ${formattedOptions.join('\n    ')}\n`);
        });
    }
    formatExamples() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { strong, weak, input } = this.colors;
            const metadata = yield this.getCommandMetadata();
            const fullName = yield this.getCommandFullName();
            if (!metadata.exampleCommands || !Array.isArray(metadata.exampleCommands)) {
                return '';
            }
            const exampleLines = metadata.exampleCommands.map(cmd => {
                const sepIndex = cmd.indexOf(' -- ');
                cmd = sepIndex === -1 ? input(cmd) : input(cmd.substring(0, sepIndex)) + cmd.substring(sepIndex);
                const wrappedCmd = format_1.wordWrap(cmd, { indentation: 12, append: ' \\' });
                return `${weak('$')} ${input(fullName)}${wrappedCmd ? ' ' + wrappedCmd : ''}`;
            });
            return (`\n  ${strong('Examples')}:` +
                `\n\n    ${exampleLines.join('\n    ')}\n`);
        });
    }
    format() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return ((yield this.formatHeader()) +
                (yield this.formatUsage()) +
                (yield this.formatInputs()) +
                (yield this.formatOptions()) +
                (yield this.formatExamples()) +
                '\n');
        });
    }
}
exports.CommandHelpFormatter = CommandHelpFormatter;
